/*
 * Harbour Project source code:
 * TDecode class
 *
 * Copyright 2001-2003 Matteo Baccan <baccan@infomedia.it>
 * www - http://www.harbour-project.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA (or visit the web site http://www.gnu.org/).
 *
 * As a special exception, the Harbour Project gives permission for
 * additional uses of the text contained in its release of Harbour.
 *
 * The exception is that, if you link the Harbour libraries with other
 * files to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the Harbour library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * This exception applies only to the code released by the Harbour
 * Project under the name Harbour.  If you copy code from other
 * Harbour Project or Free Software Foundation releases into a copy of
 * Harbour, as the General Public License permits, the exception does
 * not apply to the code that you add in this way.  To avoid misleading
 * anyone as to the status of such modified files, you must delete
 * this exception notice from them.
 *
 * If you write modifications of your own for Harbour, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.
 *
 */

#include "common.ch"
#include "hbclass.ch"

CLASS TDecode

   METHOD New()

   METHOD Decode( cString )
   METHOD Encode( cString )

   METHOD Encode64( cData )

   // TODO
   //METHOD Decode64( cData )

   // Buffer common to all classes
   CLASSDATA cCharPos init "0123456789abcdef" HIDDEN
   CLASSDATA Base64   init "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/" HIDDEN

ENDCLASS

//
// New
//
METHOD New CLASS TDecode
//::cCharPos := "0123456789abcdef"
return Self

//
// Decode String
//
METHOD Decode( cString ) CLASS TDecode
LOCAL cRet := ""
LOCAL nChars, nDec, i, cChar
LOCAL nPos1, nPos2

i = 1
nChars = Len(cString)

While i < (nChars + 1)
   cChar = substr( cString , i  , 1 )
   if cChar=="+"
      cRet += " "
   elseif cChar=="%"
      cChar := lower(substr( cString , i + 1 , 2 ))
      nPos1 := AT( Left(cChar,1), ::cCharPos )-1
      nPos2 := AT( Right(cChar,1), ::cCharPos )-1
      nDec := (nPos1*16) +nPos2
      cRet += CHR( nDec )
      i+=2
   else
      cRet += cChar
   endif
   i++
enddo

return cRet

//
// Encode String
//
METHOD Encode( cString ) CLASS TDecode
LOCAL cRet := ""
LOCAL nChars, nChar, i, cChar

i := 1
nChars = Len(cString)

While i < (nChars + 1)
   cChar = substr( cString , i  , 1 )
   if cChar==" "
      cRet += "+"
   Elseif cChar>="a" .And. cChar<="z"
      cRet += cChar
   Elseif cChar>="A" .And. cChar<="Z"
      cRet += cChar
   Elseif cChar>="0" .And. cChar<="9"
      cRet += cChar
   Else
      nChar = Asc(cChar)
      cRet += "%" +substr( ::cCharPos, Int(nChar/16)+1, 1 ) +substr( ::cCharPos, nChar-Int(nChar/16)*16+1, 1 )
   EndIf
   i++
enddo

return cRet

//
// Base64 String encode
//
METHOD Encode64( cData, bRet ) CLASS TDecode
LOCAL cOut, sOut:="", I, nGroup, pOut, sGroup

DEFAULT bRet TO .T.

// For each group of 3 bytes
for i = 1 to len(cData) Step 3

   // Long from 3 bytes.
   nGroup = 256*256*asc(substr(cData, I, 1)) +256 * asc2(substr(cData, I + 1, 1)) + asc2(substr(cData, I + 2, 1))

   // Create octal
   nGroup = DECTOOCTAL(nGroup)

   // Pad with zero
   nGroup = PADL(nGroup,8,"0")

   // base64
   pOut = substr(::Base64, OCTALTODEC( substr(nGroup, 1, 2)) + 1, 1) +;
          substr(::Base64, OCTALTODEC( substr(nGroup, 3, 2)) + 1, 1) +;
          substr(::Base64, OCTALTODEC( substr(nGroup, 5, 2)) + 1, 1) +;
          substr(::Base64, OCTALTODEC( substr(nGroup, 7, 2)) + 1, 1)

   // Add to output
   sOut += pOut

   if bRet
      // New line every 76 chars (76*3/4 = 57) or 60 (60*3/4 = 45)
      if (i + 2) % 57 = 0
         sOut += CHR(13)+CHR(10)
      endif
   endif
Next

do case
   case Len(cData) % 3 == 0
   case Len(cData) % 3 == 1
      sOut = Left(sOut, Len(sOut) - 2) + "=="
   case Len(cData) % 3 == 2
      sOut = Left(sOut, Len(sOut) - 1) + "="
End case

return sOut

static function ASC2(c)

local nRet := 0

If len(c)==1
   nRet = Asc(c)
endif

return nRet

// Gram From harbour\contrib\libmisc\nconvert.prg
STATIC FUNCTION OctalToDec(cString)
   local nNumber:=0,nX:=0
   local cNewString:=AllTrim(cString)
   local nLen:=Len(cNewString)
   FOR nX:=1 to nLen
      nNumber+=(At(SubStr(cNewString,nX,1),'01234567')-1)*;
         (8**(nLen-nX))
   NEXT nX
   RETURN(nNumber)

// Gram From harbour\contrib\libmisc\nconvert.prg
STATIC FUNCTION DecToOctal(nNumber)
   local cNewString:=''
   local nTemp:=0
   WHILE(nNumber > 0)
      nTemp:=(nNumber%8)
      cNewString:=SubStr('01234567',(nTemp+1),1)+cNewString
      nNumber:=Int((nNumber-nTemp)/8)
   ENDDO
   RETURN(cNewString)
