/*
 * Harbour Project source code:
 * THttp class
 *
 * Copyright 2001-2003 Matteo Baccan <baccan@infomedia.it>
 * www - http://www.harbour-project.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA (or visit the web site http://www.gnu.org/).
 *
 * As a special exception, the Harbour Project gives permission for
 * additional uses of the text contained in its release of Harbour.
 *
 * The exception is that, if you link the Harbour libraries with other
 * files to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the Harbour library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * This exception applies only to the code released by the Harbour
 * Project under the name Harbour.  If you copy code from other
 * Harbour Project or Free Software Foundation releases into a copy of
 * Harbour, as the General Public License permits, the exception does
 * not apply to the code that you add in this way.  To avoid misleading
 * anyone as to the status of such modified files, you must delete
 * this exception notice from them.
 *
 * If you write modifications of your own for Harbour, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.
 *
 */

#include "common.ch"
#include "hbclass.ch"

CLASS THttp

   METHOD New()

   METHOD Connect( cAddress, nPort )
   METHOD Close()

   METHOD SetProxy( cProxy, nPort, cUser, cPwd )

   METHOD Get( cURL, aValue )
   METHOD Post( cURL, aValue )

   METHOD Value2String( aValue )
   
   VAR oSocket       HIDDEN
   VAR cProxyAddress HIDDEN
   VAR nProxyPort    HIDDEN
   VAR cLogin        HIDDEN
   VAR cHostAddress  HIDDEN
   VAR nHostPort     HIDDEN

ENDCLASS


METHOD New() CLASS THttp
::oSocket := TSocket():New()
return Self

//
// Connect to remore site
//
METHOD Connect( cAddress, nPort ) CLASS THttp
LOCAL lRet := .F.

DEFAULT nPort TO 80
::cHostAddress := cAddress
::nHostPort    := nPort

IF ::cProxyAddress!=NIL
   lRet := ::oSocket:Connect(::cProxyAddress,::nProxyPort)
ELSE
   lRet := ::oSocket:Connect(::cHostAddress,::nHostPort)
ENDIF

return lRet

//
// Close socket
//
METHOD Close() CLASS THttp
return ::oSocket:Close()

//
// Send string to socket
//
METHOD Get( cPage, aPair ) CLASS THttp
LOCAL cRet := ""
LOCAL cURL := ""        
LOCAL cPost := ::Value2String( aPair )

cURL := "GET "
IF ::cProxyAddress!=NIL
   cURL += "http://" +::cHostAddress +":" +ALLTRIM(STR(::nHostPort))
ENDIF
cURL += cPage 
if len(cPost)>0
   cURL += "?" +cPost
endif
cURL += " HTTP/1.0" +CHR(13)+CHR(10) 
if ::cLogin!=NIL
   cURL += "Proxy-authorization: Basic " +::cLogin +CHR(13)+CHR(10)
endif
cURL += CHR(13)+CHR(10)

IF ::oSocket:SendString( cURL )
   cRet := ::oSocket:ReceiveString()
ENDIF
return cRet

//
// Post string to socket
//
METHOD Post( cPage, aPair ) CLASS THttp
LOCAL cRet := ""
LOCAL cURL := ""
LOCAL cPost := ::Value2String( aPair )

cURL := "POST "
IF ::cProxyAddress!=NIL
   cURL += "http://" +::cHostAddress +":" +ALLTRIM(STR(::nHostPort))
ENDIF
cURL += cPage +" HTTP/1.0" +CHR(13)+CHR(10)
cURL += "Content-Length: " +ALLTRIM(STR(LEN(cPost))) +CHR(13)+CHR(10)
cURL += "Content-Type: application/x-www-form-urlencoded" +CHR(13)+CHR(10)
if ::cLogin!=NIL
   cURL += "Proxy-authorization: Basic " +::cLogin +CHR(13)+CHR(10)
endif
cURL += CHR(13)+CHR(10)
cURL += cPost

IF ::oSocket:SendString( cURL )
   cRet := ::oSocket:ReceiveString()
ENDIF
return cRet

METHOD Value2String( aPair ) CLASS THttp
local cPost := ""
LOCAL nPair := 0
LOCAL oDecode := TDecode():new()

DEFAULT aPair TO {}

for nPair := 1 to len( aPair )
   if nPair!=1
      cPost += "&"
   endif
   cPost += aPair[nPair]:cKey +"=" +oDecode:encode(aPair[nPair]:cValue)
next

return cPost

//
// Set proxy port
//
METHOD SetProxy( cProxy, nPort, cUser, cPwd ) CLASS THttp
local oSock
DEFAULT nPort TO 8080
::cProxyAddress := cProxy
::nProxyPort    := nPort
if cUser!=NIL 
   oSock := TDecode():new()
   if cPwd!=nil
      ::cLogin := oSock:encode64( cUser+":"+cPwd )
   else
      ::cLogin := oSock:encode64( cUser )
   endif
else 
   ::cLogin := NIL
endif
return 
