/*
 * Harbour Project source code:
 * TSMTP class
 *
 * Copyright 2001-2003 Matteo Baccan <baccan@infomedia.it>
 * www - http://www.harbour-project.org
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this software; see the file COPYING.  If not, write to
 * the Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307 USA (or visit the web site http://www.gnu.org/).
 *
 * As a special exception, the Harbour Project gives permission for
 * additional uses of the text contained in its release of Harbour.
 *
 * The exception is that, if you link the Harbour libraries with other
 * files to produce an executable, this does not by itself cause the
 * resulting executable to be covered by the GNU General Public License.
 * Your use of that executable is in no way restricted on account of
 * linking the Harbour library code into it.
 *
 * This exception does not however invalidate any other reasons why
 * the executable file might be covered by the GNU General Public License.
 *
 * This exception applies only to the code released by the Harbour
 * Project under the name Harbour.  If you copy code from other
 * Harbour Project or Free Software Foundation releases into a copy of
 * Harbour, as the General Public License permits, the exception does
 * not apply to the code that you add in this way.  To avoid misleading
 * anyone as to the status of such modified files, you must delete
 * this exception notice from them.
 *
 * If you write modifications of your own for Harbour, it is your choice
 * whether to permit this exception to apply to your modifications.
 * If you do not wish that, delete this exception notice.
 *
 */

#include "common.ch"
#include "hbclass.ch"

// RFC 821
CLASS TSMTP

   METHOD New()

   METHOD Connect( cAddress, nPort )
   METHOD Login( cUser, cPwd )
   METHOD Close()

   METHOD ClearData()
   METHOD SetFrom( cUser, cEmail )
   METHOD SetSubject( cSubject )

   METHOD AddTo( cUser, cEmail )
   METHOD AddCc( cUser, cEmail )
   METHOD AddBcc( cUser, cEmail )

   METHOD SetData( cMail )

   METHOD AddAttach( cAttach )

   METHOD Send( bIgnoreTOError )
   METHOD GetLastError()

   VAR oSocket  HIDDEN
   VAR cFrom    HIDDEN
   VAR cEmail   HIDDEN
   VAR cSubject HIDDEN
   VAR aTo      HIDDEN
   VAR aCc      HIDDEN
   VAR aBcc     HIDDEN
   VAR cData    HIDDEN
   VAR bHTML    HIDDEN
   VAR aAttach  HIDDEN
   VAR cError   HIDDEN

ENDCLASS


METHOD New() CLASS TSMTP
::oSocket := TSocket():New()
//::oSocket:SetDebug( .T. )
::ClearData()
return Self

//
// Connect to remore site
//
METHOD Connect( cAddress, nPort, cHelo ) CLASS TSMTP
local bRet, cErr, bExit

DEFAULT nPort TO 25
DEFAULT cHelo TO "Harbour-TSMTP"

//## may be better use tsocket():getlocalname() for cHelo default

bRet := ::oSocket:Connect(cAddress,nPort)
// If connect read banner string
if bRet
   // Consume banner
   cErr := ::oSocket:ReceiveLine()
   if LEFT(cErr,3)=="220"
      // Login
      if ::oSocket:SendString( "HELO " +cHelo +CHR(13)+CHR(10) )
         bExit := .F.
         while !bExit
            // Banner
            cErr := ::oSocket:ReceiveLine()
            if !(LEFT(cErr,3)=="250")
               ::cError := cErr
               bRet  := .F.
               bExit := .T.
            elseif LEFT(cErr,4)=="250-"
               // Multiline header .. read next line
            else
               // Exit from loop
               bExit := .T.
            endif
         enddo
      endif
   else
      ::cError := cErr
      bRet := .F.
   endif
endif
return bRet

// Login to server
METHOD Login( cUser, cPwd ) CLASS TSMTP
local cRet    := ""
local bRet    := .F.
local oDecode := TDecode():new()

if ::oSocket:SendString( "AUTH LOGIN" +CHR(13)+CHR(10) )
   // Consume banner
   cErr := ::oSocket:ReceiveLine()
   if LEFT(cErr,3)=="334"
      if ::oSocket:SendString( oDecode:Encode64( cUser ) +CHR(13)+CHR(10) )
         // Consume banner
         cErr := ::oSocket:ReceiveLine()
         if LEFT(cErr,3)=="334"
            if ::oSocket:SendString( oDecode:Encode64( cPwd ) +CHR(13)+CHR(10) )
               // Consume banner
               cErr := ::oSocket:ReceiveLine()
               if LEFT(cErr,3)=="235"
                  bRet := .T.
               endif
            endif
         endif
      endif
   endif
endif

if !bRet
   ::cError := cErr
endif

return bRet



//
// Close socket
//
METHOD Close() CLASS TSMTP
::oSocket:SendString( "QUIT" +CHR(13)+CHR(10) )
//## check if is ok
::oSocket:ReceiveLine()
return ::oSocket:Close()

//
// Clear data
//
METHOD ClearData() CLASS TSMTP
::cFrom   := ""
::cEmail  := ""
::aTo     := {}
::aCc     := {}
::aBcc    := {}
::cData   := ""
::bHTML   := .f.
::aAttach := {}
::cError  := ""
return nil

//
// Set From
//
METHOD SetFrom( cUser, cEmail )  CLASS TSMTP
::cFrom  := cUser
::cEmail := cEmail
return nil

//
// Set Subject
//
METHOD SetSubject( cSubject )  CLASS TSMTP
::cSubject := cSubject
return nil

//
// Add to
//
METHOD AddTo( cUser, cEmail ) CLASS TSMTP
aadd( ::aTo, {cUser,cEmail} )
return nil

//
// Add cc
//
METHOD AddCc( cUser, cEmail ) CLASS TSMTP
aadd( ::aCc, {cUser,cEmail} )
return nil

//
// Add Bcc
//
METHOD AddBcc( cUser, cEmail ) CLASS TSMTP
aadd( ::aBcc, {cUser,cEmail} )
return nil

//
// Set data
//
METHOD SetData( cMail, bHTML ) CLASS TSMTP
DEFAULT bHTML TO .f.
::cData := cMail
::bHTML := bHTML
return nil

//
// Add attach
//
METHOD AddAttach( cAttach ) CLASS TSMTP
aadd( ::aAttach, cAttach )
return nil

//
// Get Error
//
METHOD GetLastError() CLASS TSMTP
return ::cError

//
// Send message
//
METHOD Send( bIgnoreTOError ) CLASS TSMTP
local bRet := .f.
local cHeader := ""
local cErr := ""
local bMultipart := (LEN(::aAttach)>0)
local cMultipart := "----=_NextPart_000_0052_01C2F554.33353C20"
local oDecode := TDecode():new()
local nPos, aEmails, bMail

DEFAULT bIgnoreTOError TO .F.

::cError := ""
if ::oSocket:SendString( "MAIL FROM: " +::cEmail +CHR(13)+CHR(10) )
   // Banner
   cErr := ::oSocket:ReceiveLine()
   // Check 250
   if left(cErr,3)=="250"

      aEmails := array(0)
      AEVAL( ::aTO,  {|aSub|AADD( aEmails, aSub[2] )} )
      AEVAL( ::aCC,  {|aSub|AADD( aEmails, aSub[2] )} )
      AEVAL( ::aBCC, {|aSub|AADD( aEmails, aSub[2] )} )

      bMail := .T.
      for nPos := 1 to len(aEmails)
         if bMail
            ::oSocket:SendString( "RCPT TO: " +aEmails[nPos] +CHR(13)+CHR(10) )
            cErr := ::oSocket:ReceiveLine()
            if !(LEFT(cErr,3)=="250") .and. !bIgnoreTOError
               ::cError := cErr
               bMail := .F.
            endif
         endif
      next

      if bMail
         // If all is OK I can prepare and send data
         if ::oSocket:SendString( "DATA" +CHR(13)+CHR(10) )
            // Banner
            cErr := ::oSocket:ReceiveLine()
            // Check 354
            if LEFT(cErr,3)=="354"
               //## to correct .. not corrext +0100 ...
               cHeader := "Date: "+left(cdow(date()),3)+", "+trans(day(date()),"99 ");
                     +trans(cmonth(date()),"AAA ")+trans(Year(date()),"9999 ")+time();
                     +" +0100" +CHR(13)+CHR(10)
               //Date: Sat, 14 Aug 2004 14:18:08 +0100

               cHeader += "From: "     +::cFrom +" " +::cEmail +CHR(13)+CHR(10)
               cHeader += "Reply-To: " +::cFrom +" " +::cEmail +CHR(13)+CHR(10)

               cHeader += addAddress( ::aTO, "To: " )
               cHeader += addAddress( ::aCC, "CC: " )
               cHeader += addAddress( ::aBCC, "BCC: " )

               //AEVAL( ::aTO , {|aSub,nPos|cHeader += "To: "  +aSub[2] +CHR(13)+CHR(10)} )
               //AEVAL( ::aCC,  {|aSub,nPos|cHeader += "CC: "  +aSub[2] +CHR(13)+CHR(10)} )
               //AEVAL( ::aCC,  {|aSub,nPos|cHeader += "BCC: "  +aSub[2] +CHR(13)+CHR(10)} )
               cHeader += "Subject: "  +::cSubject +CHR(13)+CHR(10)

               //## add properties to modify it
               cHeader += "X-Mailer: Harbour TSMTP by Matteo Baccan" +CHR(13)+CHR(10)

               cHeader += "Mime-Version: 1.0" +CHR(13)+CHR(10)
               if bMultipart .or. ::bHTML
                  cHeader += "Content-Type: multipart/mixed;" +CHR(13)+CHR(10)
                  cHeader += [        boundary="] +cMultipart +["] +CHR(13)+CHR(10)

                  // Empty line
                  cHeader += "" +CHR(13)+CHR(10)
                  cHeader += "This is a multi-part message in MIME format." +CHR(13)+CHR(10)
                  cHeader += "" +CHR(13)+CHR(10)

                  // Message
                  cHeader += "--" +cMultipart +CHR(13)+CHR(10)
                  if ::bHTML
                     cHeader += "Content-Type: text/html;" +CHR(13)+CHR(10)
                  else
                     cHeader += "Content-Type: text/plain;" +CHR(13)+CHR(10)
                     cHeader += [        charset="iso-8859-1"] +CHR(13)+CHR(10)
                  endif
                  cHeader += "Content-Transfer-Encoding: base64" +CHR(13)+CHR(10)
                  cHeader += "" +CHR(13)+CHR(10)
                  cHeader += oDecode:Encode64( ::cData ) +CHR(13)+CHR(10)
                  cHeader += "" +CHR(13)+CHR(10)

                  // Attach
                  for nPos := 1 to len(::aAttach)
                     cHeader += "--" +cMultipart +CHR(13)+CHR(10)
                     cHeader += "Content-Type: application/octet-stream;" +CHR(13)+CHR(10)
                     cHeader += [        name="] +::aAttach[nPos] +["] +CHR(13)+CHR(10)
                     cHeader += "Content-Transfer-Encoding: base64" +CHR(13)+CHR(10)
                     cHeader += "Content-Disposition: attachment;" +CHR(13)+CHR(10)
                     cHeader += [        filename="] +::aAttach[nPos] +["] +CHR(13)+CHR(10)
                     cHeader += "" +CHR(13)+CHR(10)
                     cHeader += oDecode:Encode64( memoread(::aAttach[nPos]) ) +CHR(13)+CHR(10)
                     cHeader += "" +CHR(13)+CHR(10)
                  next

                  // End of mail
                  cHeader += "--" +cMultipart +"--"
               else
                  cHeader += "Content-Type: text/plain; charset=us-ascii" +CHR(13)+CHR(10)
                  cHeader += "Content-Transfer-Encoding: 7bit" +CHR(13)+CHR(10)

                  // Empty line
                  cHeader += "" +CHR(13)+CHR(10)

                  // Data
                  cHeader += ::cData
               endif

               // End of mail
               cHeader += CHR(13)+CHR(10) +"." +CHR(13)+CHR(10)

               if ::oSocket:SendString( cHeader )
                  cErr := ::oSocket:ReceiveLine()
                  if !(LEFT(cErr,3)=="250")
                     ::cError := cErr
                  else
                     bRet := .t.
                  endif
               endif
            else
               ::cError := cErr
            endif
         endif
      endif
   else
      ::cError := cErr
   endif

endif

return bRet

* 
STATIC FUNCTION addAddress( aEmail, cTok )
* 
local cRet := ""

if len( aEmail )>0
   cRet += cTok
   AEVAL( aEmail,  {|aSub,nPos|cRet += if(nPos==1,"",","+CHR(13)+CHR(10)+"   ") +aSub[2]} )
   cRet += CHR(13)+CHR(10)
endif

RETURN cRet


/*

HELO SatanCity
MAIL FROM:<mkuser1@localhost>
RCPT TO:<mkuser1@localhost>
DATA
Date: Tue, 18 Dec 2001 21:24:08 +0100
From: mkuser1 <mkuser1@localhost>
Reply-To: mkuser1 <mkuser1@localhost>
To: mkuser1@localhost
Organization: localhost
Subject:
X-Mailer: FARMail v2.3 (build 25)
Mime-Version: 1.0
Content-Type: text/plain; charset=us-ascii
Content-Transfer-Encoding: 7bit

nohup java sockRedirector &

.
QUIT


220 <1008707048.2020@SatanCity> [XMail 1.2 (Win32/Ix86) ESMTP Server] service ready; Tue, 18 Dec 2001 21:24:08 +0100
250 xmailserver.test
250 OK
250 OK
354 Start mail input; end with <CRLF>.<CRLF>
250 OK <S5>
221 [XMail 1.2 (Win32/Ix86) ESMTP Server] service closing transmission channel
*/
